--- 2010 SRC "stereomaster.vhd" VHDL Code 
--- Current file name: stereo.vhd
--- Last Revised:  10/27/2008; 9:45 a.m.
--- Author:  WDR
--- Copyright: William D. Richard, Ph.D.

LIBRARY IEEE ;
USE IEEE.STD_LOGIC_1164.ALL ;
USE IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY stereomaster IS
   PORT (d        : IN    STD_LOGIC_VECTOR(31 DOWNTO 0) ;
         address  : INOUT STD_LOGIC_VECTOR(31 DOWNTO 0) ;
         read     : INOUT STD_LOGIC ;
         done     : INOUT STD_LOGIC ;
         write    : IN    STD_LOGIC ;
         reset_l  : IN    STD_LOGIC ;
         left     : OUT   STD_LOGIC_VECTOR(15 DOWNTO 0) ;
         right    : OUT   STD_LOGIC_VECTOR(15 DOWNTO 0) ;
         grant1   : IN    STD_LOGIC ;
         request1 : OUT   STD_LOGIC ;
         clk      : IN    STD_LOGIC) ;
END stereomaster ;

ARCHITECTURE behavioral OF stereomaster IS

   SIGNAL counter             : UNSIGNED(5 DOWNTO 0) ;
   SIGNAL audio_osc           : STD_LOGIC ;
   SIGNAL audio_osc_z1        : STD_LOGIC ;
   SIGNAL audio_osc_z2        : STD_LOGIC ;
   SIGNAL audio_osc_z3        : STD_LOGIC ;
   SIGNAL start_address       : STD_LOGIC_VECTOR(31 DOWNTO 0) ;
   SIGNAL internal_address    : STD_LOGIC_VECTOR(31 DOWNTO 0) ;
   SIGNAL length              : STD_LOGIC_VECTOR(30 DOWNTO 0) ;
   SIGNAL word_count          : STD_LOGIC_VECTOR(30 DOWNTO 0) ;
   SIGNAL go                  : STD_LOGIC ;
   SIGNAL end_l               : STD_LOGIC ;
   SIGNAL enable              : STD_LOGIC ;
   SIGNAL oe                  : STD_LOGIC ;
   SIGNAL inc                 : STD_LOGIC ;
   SIGNAL dec                 : STD_LOGIC ;
   SIGNAL readen              : STD_LOGIC ;
   SIGNAL next_h              : STD_LOGIC ;

   TYPE states IS (s0, s1, s2, s3);
   SIGNAL state: states := s0;
   SIGNAL nxt_state : states := s0;

BEGIN

   audio_osc <= counter(5) ;

   oscprocess:PROCESS(clk)
   begin
      IF (clk = '1' AND clk'EVENT) THEN
         IF (reset_l = '0') THEN
            counter <= "000000" ;
         ELSE
            counter <= counter + 1 ;
         END IF;
      END IF;
   END PROCESS oscprocess ; 

   startprocess:PROCESS(clk)
   begin
      IF (clk = '1' AND clk'EVENT) THEN
         IF (reset_l = '0') THEN
            start_address <= (OTHERS => '0') ;
         ELSIF (address(31 DOWNTO 0) = "11111111111111111111111111111100" AND write = '1') THEN
            start_address  <= d(31 DOWNTO 0) ;
         END IF;
      END IF;
   END PROCESS startprocess ;

   lengthprocess:PROCESS(clk)
   begin
      IF (clk = '1' AND clk'EVENT) THEN
         IF (reset_l = '0') THEN
            length <= (OTHERS => '0') ;
            go <= '0' ;    
         ELSIF (address(31 DOWNTO 0) = "11111111111111111111111111111000" AND write = '1') THEN
            length  <= d(30 DOWNTO 0) ;
            go      <= d(31) ;
         END IF;
      END IF;
   END PROCESS lengthprocess ;

   done <= '1' WHEN (address(31 DOWNTO 0) = "11111111111111111111111111111100" AND write = '1') OR
                    (address(31 DOWNTO 0) = "11111111111111111111111111111000" AND write = '1')
               ELSE 'Z' ;

   dacprocess:PROCESS(clk)
   begin
      IF (clk = '1' AND clk'EVENT) THEN
         IF (enable = '1') THEN
            left  <= d(31 DOWNTO 16) ;
            right <= d(15 DOWNTO 0) ;
         END IF;
      END IF;
   END PROCESS dacprocess ; 

   addresscounter:PROCESS(clk)
   begin
      IF (clk = '1' AND clk'EVENT) THEN
         IF (go = '0') THEN
            internal_address <= start_address ;
         ELSIF (inc = '1') THEN
            internal_address <= internal_address + 4 ;
         END IF;
      END IF;
   END PROCESS addresscounter ;

   address <= internal_address WHEN oe = '1' ELSE (OTHERS => 'Z') ;

   xfercounter:PROCESS(clk)
   begin
      IF (clk = '1' AND clk'EVENT) THEN
         IF (go = '0') THEN
            word_count <= length ;
         ELSIF (dec = '1') THEN
            word_count <= word_count - 1 ;
         END IF;
      END IF;
   END PROCESS xfercounter ;

   end_l <= '0' WHEN word_count = "0000000000000000000000000000000" ELSE '1' ;

   syncflops:PROCESS(clk)
   begin
      IF (clk = '1' AND clk'EVENT) THEN
         audio_osc_z1 <= audio_osc ;
         audio_osc_z2 <= audio_osc_z1 ;
         audio_osc_z3 <= audio_osc_z2 ;
      END IF;
   END PROCESS syncflops ;

   next_h <= '1' WHEN audio_osc_z2 = '1' AND audio_osc_z3 = '0' ELSE '0' ;

   read <= '1' WHEN readen = '1' ELSE 'Z' ;

-- process to implement the state register

   clkd: PROCESS (clk)
   BEGIN
      IF (clk'EVENT AND clk='1') THEN
         IF (reset_l = '0') THEN
            state <= s0;
         ELSE
            state <= nxt_state;
         END IF;
      END IF;
    END PROCESS clkd;

-- process to determine next state

   state_trans: PROCESS (go, next_h, grant1, done, end_l, state)
   BEGIN
      nxt_state <= state ; 
      CASE state IS
         WHEN s0 => IF (go = '1' AND next_h = '1') THEN
                       nxt_state <= s1;
                    END IF ; 
         WHEN s1 => IF (grant1 = '1') THEN
                       nxt_state <= s2;
                    END IF ;
         WHEN s2 => IF (done = '1') THEN
                       nxt_state <= s3;
                    END IF ; 
         WHEN s3 => IF (end_l = '1') THEN
                       nxt_state <= s0;
                    END IF ; 
         END CASE;
   END PROCESS state_trans;

--- statment to define the output values

   request1 <= '1' WHEN (state = s1) OR (state = s2) ELSE '0' ;
   readen   <= '1' WHEN (state = s2) ELSE '0' ;
   oe       <= '1' WHEN (state = s2) ELSE '0' ;
   enable   <= '1' WHEN (state = s2) AND done = '1' ELSE '0' ;
   inc      <= '1' WHEN (state = s2) AND done = '1' ELSE '0' ;
   dec      <= '1' WHEN (state = s2) AND done = '1' ELSE '0' ;

END behavioral ;
